@@ -3,12 +3,14 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2013-2014 Michael Wiencek
-# Copyright (C) 2013-2016, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2016, 2018-2022 Laurent Monin
 # Copyright (C) 2014, 2017 Lukáš Lalinský
-# Copyright (C) 2014, 2018-2021 Philipp Wolfer
+# Copyright (C) 2014, 2018-2023 Philipp Wolfer
 # Copyright (C) 2015 Ohm Patel
 # Copyright (C) 2016 Suhas
 # Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2021 Gabriel Ferreira
+# Copyright (C) 2021, 2023 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -33,13 +35,16 @@ from picard import log
 from picard.config import (
     BoolOption,
     IntOption,
+    ListOption,
+    Option,
     TextOption,
 )
 from picard.const import (
     DEFAULT_FILE_NAMING_FORMAT,
-    DEFAULT_NUMBERED_SCRIPT_NAME,
+    DEFAULT_SCRIPT_NAME,
 )
 from picard.const.sys import IS_FROZEN
+from picard.util import unique_numbered_title
 
 
 # TO ADD AN UPGRADE HOOK:
@@ -79,7 +84,7 @@ def upgrade_to_v1_0_0_final_0(config, interactive=True, merge=True):
                         "albums has been removed in this version of Picard.\n"
                         "Your file naming scheme has automatically been "
                         "merged with that of single artist albums."),
-                    QtWidgets.QMessageBox.Ok)
+                    QtWidgets.QMessageBox.StandardButton.Ok)
 
         elif (_s.value("va_file_naming_format", TextOption)
               != r"$if2(%albumartist%,%artist%)/%album%/$if($gt(%totaldis"
@@ -94,9 +99,9 @@ def upgrade_to_v1_0_0_final_0(config, interactive=True, merge=True):
                     "separate file naming scheme defined.\n"
                     "Do you want to remove it or merge it with your file "
                     "naming scheme for single artist albums?"))
-                msgbox.setIcon(QtWidgets.QMessageBox.Question)
-                merge_button = msgbox.addButton(_('Merge'), QtWidgets.QMessageBox.AcceptRole)
-                msgbox.addButton(_('Remove'), QtWidgets.QMessageBox.DestructiveRole)
+                msgbox.setIcon(QtWidgets.QMessageBox.Icon.Question)
+                merge_button = msgbox.addButton(_('Merge'), QtWidgets.QMessageBox.ButtonRole.AcceptRole)
+                msgbox.addButton(_('Remove'), QtWidgets.QMessageBox.ButtonRole.DestructiveRole)
                 msgbox.exec_()
                 merge = msgbox.clickedButton() == merge_button
             remove_va_file_naming_format(merge=merge)
@@ -224,7 +229,7 @@ def upgrade_to_v1_4_0_dev_6(config):
     if old_script_text_option in _s:
         old_script_text = _s.value(old_script_text_option, TextOption, "")
         if old_script_text:
-            old_script = (0, _(DEFAULT_NUMBERED_SCRIPT_NAME) % 1, _s["enable_tagger_scripts"], old_script_text)
+            old_script = (0, unique_numbered_title(_(DEFAULT_SCRIPT_NAME), list_of_scripts), _s["enable_tagger_scripts"], old_script_text)
             list_of_scripts.append(old_script)
     _s["list_of_scripts"] = list_of_scripts
     _s.remove(old_enabled_option)
@@ -301,7 +306,9 @@ def upgrade_to_v2_4_0_beta_3(config):
     """Convert preserved tags to list"""
     _s = config.setting
     opt = 'preserved_tags'
-    _s[opt] = [t.strip() for t in _s.raw_value(opt, qtype='QString').split(',')]
+    value = _s.raw_value(opt, qtype='QString')
+    if not isinstance(value, list):
+        _s[opt] = [t.strip() for t in value.split(',')]
 
 
 def upgrade_to_v2_5_0_dev_1(config):
@@ -341,12 +348,142 @@ def upgrade_to_v2_6_0_beta_3(config):
     _s.remove("use_system_theme")
 
 
+def upgrade_to_v2_7_0_dev_2(config):
+    """Replace manually set persistent splitter settings with automated system.
+    """
+    def upgrade_persisted_splitter(new_persist_key, key_map):
+        _p = config.persist
+        splitter_dict = {}
+        for (old_splitter_key, new_splitter_key) in key_map:
+            if _p.__contains__(old_splitter_key):
+                if _p[old_splitter_key] is not None:
+                    splitter_dict[new_splitter_key] = bytearray(_p[old_splitter_key])
+                _p.remove(old_splitter_key)
+        Option("persist", new_persist_key, {})
+        _p[new_persist_key] = splitter_dict
+
+    # MainWindow splitters
+    upgrade_persisted_splitter(
+        new_persist_key="splitters_MainWindow",
+        key_map=[
+            ('bottom_splitter_state', 'main_window_bottom_splitter'),
+            ('splitter_state', 'main_panel_splitter'),
+        ]
+    )
+
+    # ScriptEditorDialog splitters
+    upgrade_persisted_splitter(
+        new_persist_key="splitters_ScriptEditorDialog",
+        key_map=[
+            ('script_editor_splitter_samples', 'splitter_between_editor_and_examples'),
+            ('script_editor_splitter_samples_before_after', 'splitter_between_before_and_after'),
+            ('script_editor_splitter_documentation', 'splitter_between_editor_and_documentation'),
+        ]
+    )
+
+    # OptionsDialog splitters
+    upgrade_persisted_splitter(
+        new_persist_key="splitters_OptionsDialog",
+        key_map=[
+            ('options_splitter', 'dialog_splitter'),
+            ('scripting_splitter', 'scripting_options_splitter'),
+        ]
+    )
+
+
+def upgrade_to_v2_7_0_dev_3(config):
+    """Save file naming scripts to dictionary.
+    """
+    from picard.script import get_file_naming_script_presets
+    from picard.script.serializer import (
+        FileNamingScript,
+        ScriptImportError,
+    )
+    Option("setting", "file_renaming_scripts", {})
+    ListOption("setting", "file_naming_scripts", [])
+    TextOption("setting", "file_naming_format", DEFAULT_FILE_NAMING_FORMAT)
+    TextOption("setting", "selected_file_naming_script_id", "")
+    scripts = {}
+    for item in config.setting["file_naming_scripts"]:
+        try:
+            script_item = FileNamingScript().create_from_yaml(item, create_new_id=False)
+            scripts[script_item["id"]] = script_item.to_dict()
+        except ScriptImportError:
+            log.error("Error converting file naming script")
+    script_list = set(scripts.keys()) | set(map(lambda item: item["id"], get_file_naming_script_presets()))
+    if config.setting["selected_file_naming_script_id"] not in script_list:
+        script_item = FileNamingScript(
+            script=config.setting["file_naming_format"],
+            title=_("Primary file naming script"),
+            readonly=False,
+            deletable=True,
+        )
+        scripts[script_item["id"]] = script_item.to_dict()
+        config.setting["selected_file_naming_script_id"] = script_item["id"]
+    config.setting["file_renaming_scripts"] = scripts
+    config.setting.remove("file_naming_scripts")
+    config.setting.remove("file_naming_format")
+
+
+def upgrade_to_v2_7_0_dev_4(config):
+    """Replace artist_script_exception with artist_script_exceptions"""
+    _s = config.setting
+    ListOption("setting", "artist_script_exceptions", [])
+    if _s["artist_script_exception"]:
+        _s["artist_script_exceptions"] = [_s["artist_script_exception"]]
+    _s.remove("artist_script_exception")
+    ListOption("setting", "artist_locales", ['en'])
+    if _s["artist_locale"]:
+        _s["artist_locales"] = [_s["artist_locale"]]
+    _s.remove("artist_locale")
+
+
+def upgrade_to_v2_7_0_dev_5(config):
+    """Replace artist_script_exceptions with script_exceptions and remove artist_script_exception_weighting"""
+    _s = config.setting
+    ListOption("setting", "script_exceptions", [])
+    weighting = _s["artist_script_exception_weighting"] or 0
+    artist_script_exceptions = _s["artist_script_exceptions"] or []
+    _s["script_exceptions"] = [(script_exception, weighting) for script_exception in artist_script_exceptions]
+    _s.remove("artist_script_exceptions")
+    _s.remove("artist_script_exception_weighting")
+
+
+def upgrade_to_v2_8_0_dev_2(config):
+    """Remove AcousticBrainz settings from options"""
+    toolbar_layout = config.setting['toolbar_layout']
+    try:
+        toolbar_layout.remove('extract_and_submit_acousticbrainz_features_action')
+        config.setting['toolbar_layout'] = toolbar_layout
+    except ValueError:
+        pass
+
+
+def upgrade_to_v2_9_0_alpha_2(config):
+    """Add preset file naming scripts to editable user scripts disctionary"""
+    from picard.script import get_file_naming_script_presets
+    scripts = config.setting["file_renaming_scripts"]
+    for item in get_file_naming_script_presets():
+        scripts[item["id"]] = item.to_dict()
+    config.setting["file_renaming_scripts"] = scripts
+
+
 def rename_option(config, old_opt, new_opt, option_type, default):
     _s = config.setting
     if old_opt in _s:
         _s[new_opt] = _s.value(old_opt, option_type, default)
         _s.remove(old_opt)
 
+        _p = config.profiles
+        _s.init_profile_options()
+        all_settings = _p["user_profile_settings"]
+        for profile in _p["user_profiles"]:
+            id = profile["id"]
+            if id in all_settings and old_opt in all_settings[id]:
+                all_settings[id][new_opt] = all_settings[id][old_opt]
+                all_settings[id].pop(old_opt)
+        _p["user_profile_settings"] = all_settings
+
 
 def upgrade_config(config):
     cfg = config
@@ -370,4 +507,10 @@ def upgrade_config(config):
     cfg.register_upgrade_hook(upgrade_to_v2_6_0_dev_1)
     cfg.register_upgrade_hook(upgrade_to_v2_6_0_beta_2)
     cfg.register_upgrade_hook(upgrade_to_v2_6_0_beta_3)
+    cfg.register_upgrade_hook(upgrade_to_v2_7_0_dev_2)
+    cfg.register_upgrade_hook(upgrade_to_v2_7_0_dev_3)
+    cfg.register_upgrade_hook(upgrade_to_v2_7_0_dev_4)
+    cfg.register_upgrade_hook(upgrade_to_v2_7_0_dev_5)
+    cfg.register_upgrade_hook(upgrade_to_v2_8_0_dev_2)
+    cfg.register_upgrade_hook(upgrade_to_v2_9_0_alpha_2)
     cfg.run_upgrade_hooks(log.debug)
