@@ -4,10 +4,10 @@
 #
 # Copyright (C) 2007 Oliver Charles
 # Copyright (C) 2007, 2010-2011 Lukáš Lalinský
-# Copyright (C) 2007-2011, 2015, 2018-2021 Philipp Wolfer
+# Copyright (C) 2007-2011, 2015, 2018-2023 Philipp Wolfer
 # Copyright (C) 2011 Michael Wiencek
 # Copyright (C) 2011-2012 Wieland Hoffmann
-# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2022 Laurent Monin
 # Copyright (C) 2015-2016 Rahul Raturi
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Frederik “Freso” S. Olesen
@@ -52,10 +52,7 @@ from picard.config import (
     ListOption,
     get_config,
 )
-from picard.const import (
-    CAA_HOST,
-    CAA_PORT,
-)
+from picard.const import CAA_URL
 from picard.coverart.image import (
     CaaCoverArtImage,
     CaaThumbnailCoverArtImage,
@@ -90,10 +87,10 @@ _CAA_THUMBNAIL_SIZE_ALIASES = {
 _CAA_IMAGE_SIZE_DEFAULT = 500
 
 _CAA_IMAGE_TYPE_DEFAULT_INCLUDE = ['front']
-_CAA_IMAGE_TYPE_DEFAULT_EXCLUDE = ['raw/unedited', 'watermark']
+_CAA_IMAGE_TYPE_DEFAULT_EXCLUDE = ['matrix/runout', 'raw/unedited', 'watermark']
 
-ratecontrol.set_minimum_delay((CAA_HOST, CAA_PORT), 0)
-ratecontrol.set_minimum_delay(('archive.org', 443), 0)
+ratecontrol.set_minimum_delay_for_url(CAA_URL, 0)
+ratecontrol.set_minimum_delay_for_url('https://archive.org', 0)
 
 
 def caa_url_fallback_list(desired_size, thumbnails):
@@ -154,7 +151,7 @@ class ArrowsColumn(QtWidgets.QWidget):
         self.selection_list = selection_list
         self.ignore_list = ignore_list
         self.callback = callback
-        spacer_item = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
+        spacer_item = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
         arrows_layout = QtWidgets.QVBoxLayout()
         arrows_layout.addItem(QtWidgets.QSpacerItem(spacer_item))
         self.button_add = ArrowButton('go-next' if reverse else 'go-previous', self.move_from_ignore)
@@ -194,9 +191,9 @@ class ListBox(QtWidgets.QListWidget):
     def __init__(self, parent=None):
         super().__init__(parent=parent)
         self.setMinimumSize(QtCore.QSize(self.LISTBOX_WIDTH, self.LISTBOX_HEIGHT))
-        self.setSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.setSortingEnabled(True)
-        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
 
     def move_item(self, item, target_list):
         """Move the specified item to another listbox."""
@@ -217,7 +214,7 @@ class ListBox(QtWidgets.QListWidget):
         if callback:
             callback()
 
-    def all_items_data(self, role=QtCore.Qt.UserRole):
+    def all_items_data(self, role=QtCore.Qt.ItemDataRole.UserRole):
         for index in range(self.count()):
             yield self.item(index).data(role)
 
@@ -241,9 +238,9 @@ class CAATypesSelectorDialog(PicardDialog):
             types_exclude = []
 
         self.setWindowTitle(_("Cover art types"))
-        self.setWindowModality(QtCore.Qt.WindowModal)
+        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
         self.layout = QtWidgets.QVBoxLayout(self)
-        self.layout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
+        self.layout.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetFixedSize)
 
         # Create list boxes for dialog
         self.list_include = ListBox()
@@ -262,7 +259,7 @@ class CAATypesSelectorDialog(PicardDialog):
         instructions = QtWidgets.QLabel()
         instructions.setText(_("Please select the contents of the image type 'Include' and 'Exclude' lists."))
         instructions.setWordWrap(True)
-        instructions.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
+        instructions.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.layout.addWidget(instructions)
 
         self.arrows_include = ArrowsColumn(
@@ -311,17 +308,17 @@ class CAATypesSelectorDialog(PicardDialog):
             "use a CAA image.\n")
         )
         instructions.setWordWrap(True)
-        instructions.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
+        instructions.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.layout.addWidget(instructions)
 
         self.buttonbox = QtWidgets.QDialogButtonBox(self)
-        self.buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        self.buttonbox.setOrientation(QtCore.Qt.Orientation.Horizontal)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.AcceptRole)
+            StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.ButtonRole.AcceptRole)
         self.buttonbox.addButton(StandardButton(StandardButton.CANCEL),
-                                 QtWidgets.QDialogButtonBox.RejectRole)
+                                 QtWidgets.QDialogButtonBox.ButtonRole.RejectRole)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.HelpRole)
+            StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.ButtonRole.HelpRole)
 
         extrabuttons = [
             (N_("I&nclude all"), self.move_all_to_include_list),
@@ -331,7 +328,7 @@ class CAATypesSelectorDialog(PicardDialog):
         ]
         for label, callback in extrabuttons:
             button = QtWidgets.QPushButton(_(label))
-            self.buttonbox.addButton(button, QtWidgets.QDialogButtonBox.ActionRole)
+            self.buttonbox.addButton(button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
             button.clicked.connect(callback)
 
         self.layout.addWidget(self.buttonbox)
@@ -375,7 +372,7 @@ class CAATypesSelectorDialog(PicardDialog):
             name = caa_type['name']
             title = translate_caa_type(name)
             item = QtWidgets.QListWidgetItem(title)
-            item.setData(QtCore.Qt.UserRole, name)
+            item.setData(QtCore.Qt.ItemDataRole.UserRole, name)
             if name in includes:
                 self.list_include.addItem(item)
             elif name in excludes:
@@ -427,7 +424,7 @@ class CAATypesSelectorDialog(PicardDialog):
             types_exclude = []
         dialog = CAATypesSelectorDialog(parent, types_include, types_exclude)
         result = dialog.exec_()
-        return (dialog.get_selected_types_include(), dialog.get_selected_types_exclude(), result == QtWidgets.QDialog.Accepted)
+        return (dialog.get_selected_types_include(), dialog.get_selected_types_exclude(), result == QtWidgets.QDialog.DialogCode.Accepted)
 
 
 class ProviderOptionsCaa(ProviderOptions):
@@ -525,14 +522,14 @@ class CoverArtProviderCaa(CoverArtProvider):
         # MB web service indicates if CAA has artwork
         # https://tickets.metabrainz.org/browse/MBS-4536
         if 'cover-art-archive' not in self.release:
-            log.debug('No Cover Art Archive information for {release_id}'.format(release_id=self.release['id']))
+            log.debug('No Cover Art Archive information for %s', self.release['id'])
             return False
 
         caa_node = self.release['cover-art-archive']
         caa_has_suitable_artwork = caa_node['artwork']
 
         if not caa_has_suitable_artwork:
-            log.debug('There are no images in the Cover Art Archive for {release_id}'.format(release_id=self.release['id']))
+            log.debug('There are no images in the Cover Art Archive for %s', self.release['id'])
             return False
 
         if self.restrict_types:
@@ -561,9 +558,9 @@ class CoverArtProviderCaa(CoverArtProvider):
                 caa_has_suitable_artwork = front_in_caa or back_in_caa
 
         if not caa_has_suitable_artwork:
-            log.debug('There are no suitable images in the Cover Art Archive for {release_id}'.format(release_id=self.release['id']))
+            log.debug('There are no suitable images in the Cover Art Archive for %s', self.release['id'])
         else:
-            log.debug('There are suitable images in the Cover Art Archive for {release_id}'.format(release_id=self.release['id']))
+            log.debug('There are suitable images in the Cover Art Archive for %s', self.release['id'])
 
         return caa_has_suitable_artwork
 
@@ -581,14 +578,12 @@ class CoverArtProviderCaa(CoverArtProvider):
         return "/release/%s/" % self.metadata["musicbrainz_albumid"]
 
     def queue_images(self):
-        self.album.tagger.webservice.get(
-            CAA_HOST,
-            CAA_PORT,
-            self._caa_path,
-            self._caa_json_downloaded,
+        self.album.tagger.webservice.get_url(
+            url=CAA_URL + self._caa_path,
+            handler=self._caa_json_downloaded,
             priority=True,
             important=False,
-            cacheloadcontrol=QNetworkRequest.PreferNetwork
+            cacheloadcontrol=QNetworkRequest.CacheLoadControl.PreferNetwork,
         )
         self.album._requests += 1
         # we will call next_in_queue() after json parsing
@@ -598,11 +593,11 @@ class CoverArtProviderCaa(CoverArtProvider):
         """Parse CAA JSON file and queue CAA cover art images for download"""
         self.album._requests -= 1
         if error:
-            if not (error == QNetworkReply.ContentNotFoundError and self.ignore_json_not_found_error):
+            if not (error == QNetworkReply.NetworkError.ContentNotFoundError and self.ignore_json_not_found_error):
                 self.error('CAA JSON error: %s' % (http.errorString()))
         else:
             if self.restrict_types:
-                log.debug('CAA types: included: %s, excluded: %s' % (self.caa_types, self.caa_types_to_omit,))
+                log.debug('CAA types: included: %s, excluded: %s', self.caa_types, self.caa_types_to_omit)
             try:
                 config = get_config()
                 for image in data["images"]:
@@ -610,8 +605,7 @@ class CoverArtProviderCaa(CoverArtProvider):
                         continue
                     is_pdf = image["image"].endswith('.pdf')
                     if is_pdf and not config.setting["save_images_to_files"]:
-                        log.debug("Skipping pdf cover art : %s" %
-                                  image["image"])
+                        log.debug("Skipping pdf cover art : %s", image["image"])
                         continue
                     # if image has no type set, we still want it to match
                     # pseudo type 'unknown'
@@ -626,10 +620,10 @@ class CoverArtProviderCaa(CoverArtProvider):
                         if types and self.caa_types_to_omit:
                             types = not set(image["types"]).intersection(
                                 set(self.caa_types_to_omit))
-                        log.debug('CAA image {status}: {image_name}  {image_types}'.format(
-                            status=('accepted' if types else 'rejected'),
-                            image_name=image['image'],
-                            image_types=image['types'],)
+                        log.debug('CAA image %s: %s  %s',
+                            ('accepted' if types else 'rejected'),
+                            image['image'],
+                            image['types']
                         )
                     else:
                         types = True
