@@ -6,7 +6,8 @@
 # Copyright (C) 2014 Shadab Zafar
 # Copyright (C) 2015-2021 Laurent Monin
 # Copyright (C) 2019 Wieland Hoffmann
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2019-2020, 2022-2023 Philipp Wolfer
+# Copyright (C) 2023 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,8 +35,13 @@ import zipfile
 import zipimport
 
 from PyQt5 import QtCore
+from PyQt5.QtWidgets import (
+    QCheckBox,
+    QMessageBox,
+)
 
 from picard import log
+from picard.config import get_config
 from picard.const import (
     PLUGINS_API,
     USER_PLUGIN_DIR,
@@ -65,6 +71,35 @@ _UPDATE_SUFFIX_LEN = len(_UPDATE_SUFFIX)
 _extension_points = []
 
 
+class PluginUpdatesDialog():
+
+    def __init__(self, parent, dialog_text):
+
+        self.show_again = True
+        show_again_text = _("Perform this check again the next time you start Picard.")
+
+        self.msg = QMessageBox(parent)
+        self.msg.setIcon(QMessageBox.Icon.Information)
+        self.msg.setText(dialog_text)
+        self.msg.setWindowTitle(_("Picard Plugins Update"))
+        self.msg.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
+
+        self.cb = QCheckBox(show_again_text)
+        self.cb.setChecked(self.show_again)
+        self.cb.toggled.connect(self._set_state)
+
+        self.msg.setCheckBox(self.cb)
+        self.msg.setStandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.Cancel)
+        self.msg.setDefaultButton(QMessageBox.StandardButton.Cancel)
+
+    def _set_state(self):
+        self.show_again = not self.show_again
+
+    def show(self):
+        show_plugins_page = self.msg.exec() == QMessageBox.StandardButton.Yes
+        return show_plugins_page, self.show_again
+
+
 def is_update(path):
     return path.endswith(_UPDATE_SUFFIX)
 
@@ -212,7 +247,7 @@ class PluginManager(QtCore.QObject):
             self.handle_plugin_updates()
         # now load found plugins
         names = set()
-        for path in [os.path.join(plugindir, file) for file in os.listdir(plugindir)]:
+        for path in (os.path.join(plugindir, file) for file in os.listdir(plugindir)):
             name = _plugin_name_from_path(path)
             if name:
                 names.add(name)
@@ -301,9 +336,10 @@ class PluginManager(QtCore.QObject):
         dirpath = os.path.join(self.plugins_directory, plugin_name)
         if not os.path.isdir(dirpath):
             dirpath = None
+        filenames = {plugin_name + ext for ext in fileexts}
         filepaths = [os.path.join(self.plugins_directory, f)
                      for f in os.listdir(self.plugins_directory)
-                     if f in [plugin_name + ext for ext in fileexts]
+                     if f in filenames
                      ]
         return (dirpath, filepaths)
 
@@ -396,7 +432,7 @@ class PluginManager(QtCore.QObject):
                 elif os.path.isdir(path):
                     self._install_plugin_dir(plugin_name, path, update=update)
             except OSError as why:
-                log.error("Unable to copy plugin '%s' to %r: %s" % (plugin_name, self.plugins_directory, why))
+                log.error("Unable to copy plugin '%s' to %r: %s", plugin_name, self.plugins_directory, why)
                 return
 
             if not update:
@@ -413,13 +449,11 @@ class PluginManager(QtCore.QObject):
                 self.plugin_updated.emit(plugin_name, False)
 
     def query_available_plugins(self, callback=None):
-        self.tagger.webservice.get(
-            PLUGINS_API['host'],
-            PLUGINS_API['port'],
-            PLUGINS_API['endpoint']['plugins'],
-            partial(self._plugins_json_loaded, callback=callback),
+        self.tagger.webservice.get_url(
+            url=PLUGINS_API['urls']['plugins'],
+            handler=partial(self._plugins_json_loaded, callback=callback),
             priority=True,
-            important=True
+            important=True,
         )
 
     def is_available(self, plugin_name):
@@ -446,3 +480,59 @@ class PluginManager(QtCore.QObject):
     # pylint: disable=no-self-use
     def enabled(self, name):
         return True
+
+    def _plugins_have_new_versions(self):
+        """Compare available plugins versions with installed plugins ones
+        and yield plugin names of plugins that have new versions"""
+        if self.available_plugins is not None:
+            available_versions = {p.module_name: p.version for p in self.available_plugins}
+            for plugin in self.plugins:
+                if plugin.module_name not in available_versions:
+                    continue
+                if available_versions[plugin.module_name] > plugin.version:
+                    yield plugin.name
+
+    def check_update(self, parent=None):
+        def _display_update():
+            UPDATE_LINES_TO_SHOW = 3
+            plugins_with_updates = sorted(self._plugins_have_new_versions())
+            if plugins_with_updates:
+                file_count = len(plugins_with_updates)
+                header = '<p>' + ngettext(
+                    "There is an update available for one of your currently installed plugins:",
+                    "There are updates available for your currently installed plugins:",
+                    file_count
+                ) + '</p>'
+                footer = '<p>' + ngettext(
+                    "Do you want to update the plugin now?",
+                    "Do you want to update the plugins now?",
+                    file_count
+                ) + '</p>'
+
+                extra_file_count = file_count - UPDATE_LINES_TO_SHOW
+                if extra_file_count > 0:
+                    extra_plugins = '<p>' + ngettext(
+                        "plus {extra_file_count:,d} other plugin.",
+                        "plus {extra_file_count:,d} other plugins.",
+                        extra_file_count).format(extra_file_count=extra_file_count) + '</p>'
+                else:
+                    extra_plugins = ''
+
+                plugin_list = ''
+                for plugin_name in plugins_with_updates[:UPDATE_LINES_TO_SHOW]:
+                    plugin_list += f"<li>{plugin_name}</li>"
+
+                msg = PluginUpdatesDialog(parent, f'{header}<ul>{plugin_list}</ul>{extra_plugins}{footer}')
+
+                show_options_page, perform_check = msg.show()
+
+                config = get_config()
+                config.setting['check_for_plugin_updates'] = perform_check
+
+                if parent and show_options_page:
+                    parent.show_plugins_options_page()
+
+        if self.available_plugins is None:
+            self.query_available_plugins(_display_update)
+        else:
+            _display_update()
