@@ -2,9 +2,9 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2021 Bob Swift
-# Copyright (C) 2021 Laurent Monin
-# Copyright (C) 2021-2022 Philipp Wolfer
+# Copyright (C) 2021, 2023 Bob Swift
+# Copyright (C) 2021-2022 Laurent Monin
+# Copyright (C) 2021-2023 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -87,7 +87,7 @@ class PicardScript():
     OUTPUT_FIELDS = ('title', 'script_language_version', 'script', 'id')
 
     # Don't automatically trigger changing the `script_last_updated` property when updating these properties.
-    _last_updated_ignore_list = {'last_updated', 'readonly', 'deletable', 'id'}
+    _last_updated_ignore_list = {'last_updated', 'id'}
 
     def __init__(self, script='', title='', id=None, last_updated=None, script_language_version=None):
         """Base class for Picard script objects
@@ -216,7 +216,7 @@ class PicardScript():
         (name, ext) = os.path.splitext(filename)
         if ext and str(name).endswith('.' + ext):
             filename = name
-        log.debug('Exporting script file: %s' % filename)
+        log.debug('Exporting script file: %s', filename)
         if file_type == self._file_types()['package']:
             script_text = self.to_yaml()
         else:
@@ -250,7 +250,7 @@ class PicardScript():
         filename, file_type = QtWidgets.QFileDialog.getOpenFileName(parent, dialog_title, default_script_directory, dialog_file_types, options=options)
         if not filename:
             return None
-        log.debug('Importing script file: %s' % filename)
+        log.debug('Importing script file: %s', filename)
         try:
             with open(filename, 'r', encoding='utf8') as i_file:
                 file_content = i_file.read()
@@ -298,8 +298,6 @@ class PicardScript():
         new_object.update_script_setting(
             title=_("%s (Copy)") % self.title,
             script_language_version=SCRIPT_LANGUAGE_VERSION,
-            readonly=False,
-            deletable=True
         )
         new_object._set_new_id()
         return new_object
@@ -398,14 +396,13 @@ class FileNamingScript(PicardScript):
         script='',
         title='',
         id=None,
-        readonly=False,
-        deletable=True,
         author='',
         description='',
         license='',
         version='',
         last_updated=None,
-        script_language_version=None
+        script_language_version=None,
+        **kwargs,   # Catch additional (deprecated) arguments to avoid error in prior version config_upgrade functions.
     ):
         """Creates a Picard file naming script object.
 
@@ -413,8 +410,6 @@ class FileNamingScript(PicardScript):
             script (str): Text of the script.
             title (str): Title of the script.
             id (str): ID code for the script. Defaults to a system generated uuid.
-            readonly (bool): Identifies if the script is readonly. Defaults to False.
-            deletable (bool): Identifies if the script can be deleted from the selection list. Defaults to True.
             author (str): The author of the script. Defaults to ''.
             description (str): A description of the script, typically including type of output and any required plugins or settings. Defaults to ''.
             license (str): The license under which the script is being distributed. Defaults to ''.
@@ -423,8 +418,6 @@ class FileNamingScript(PicardScript):
             script_language_version (str): The version of the script language supported by the script.
         """
         super().__init__(script=script, title=title, id=id, last_updated=last_updated, script_language_version=script_language_version)
-        self.readonly = readonly    # for presets
-        self.deletable = deletable  # Allow removal from list of scripts
         self.author = author
         self.description = description
         self.license = license
