@@ -6,12 +6,12 @@
 # Copyright (C) 2006-2007, 2011 Lukáš Lalinský
 # Copyright (C) 2008 Gary van der Merwe
 # Copyright (C) 2009 Carlin Mangar
-# Copyright (C) 2010, 2014-2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2010, 2014-2015, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011 Chad Wilson
 # Copyright (C) 2011 Wieland Hoffmann
 # Copyright (C) 2011-2013 Michael Wiencek
 # Copyright (C) 2014, 2017 Sophist-UK
-# Copyright (C) 2014-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2014-2015, 2018-2022 Laurent Monin
 # Copyright (C) 2016 Mark Trolley
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016 Suhas
@@ -20,6 +20,9 @@
 # Copyright (C) 2018 Calvin Walton
 # Copyright (C) 2018 Vishal Choudhary
 # Copyright (C) 2019 Joel Lintunen
+# Copyright (C) 2020, 2022 Adam James
+# Copyright (C) 2021 Gabriel Ferreira
+# Copyright (C) 2021 Petit Minion
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,18 +39,17 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from collections import defaultdict
-from functools import partial
-from itertools import filterfalse
+from collections import (
+    Counter,
+    defaultdict,
+)
 import re
 import traceback
 
 from PyQt5 import QtCore
 
-from picard import (
-    config,
-    log,
-)
+from picard import log
+from picard.config import get_config
 from picard.const import (
     DATA_TRACK_TITLE,
     SILENCE_TRACK_TITLE,
@@ -61,6 +63,7 @@ from picard.file import (
 from picard.mbjson import recording_to_metadata
 from picard.metadata import (
     Metadata,
+    MultiMetadataProxy,
     run_track_metadata_processors,
 )
 from picard.script import (
@@ -68,22 +71,15 @@ from picard.script import (
     ScriptParser,
     enabled_tagger_scripts_texts,
 )
+from picard.util import pattern_as_regex
 from picard.util.imagelist import (
     ImageList,
     add_metadata_images,
     remove_metadata_images,
-    update_metadata_images,
 )
 from picard.util.textencoding import asciipunct
 
-from picard.ui.item import Item
-
-
-_TRANSLATE_TAGS = {
-    "hip hop": "Hip-Hop",
-    "synth-pop": "Synthpop",
-    "electronica": "Electronic",
-}
+from picard.ui.item import FileListItem
 
 
 class TagGenreFilter:
@@ -93,28 +89,17 @@ class TagGenreFilter:
         self.match_regexes = defaultdict(list)
         for lineno, line in enumerate(filters.splitlines()):
             line = line.strip()
-            if line and line[0] in ('+', '-'):
+            if line and line[0] in {'+', '-'}:
                 _list = line[0]
                 remain = line[1:].strip()
                 if not remain:
                     continue
-                if len(remain) > 2 and remain[0] == '/' and remain[-1] == '/':
-                    remain = remain[1:-1]
-                    try:
-                        regex_search = re.compile(remain, re.IGNORECASE)
-                    except Exception as e:
-                        log.error("Failed to compile regex /%s/: %s", remain, e)
-                        self.errors[lineno] = str(e)
-                        regex_search = None
-                else:
-                    # FIXME?: only support '*' (not '?' or '[abc]')
-                    # replace multiple '*' by one
-                    star = re.escape('*')
-                    remain = re.sub(star + '+', '*', remain)
-                    regex = '.*'.join([re.escape(x) for x in remain.split('*')])
-                    regex_search = re.compile('^' + regex + '$', re.IGNORECASE)
-                if regex_search:
+                try:
+                    regex_search = pattern_as_regex(remain, allow_wildcards=True, flags=re.IGNORECASE)
                     self.match_regexes[_list].append(regex_search)
+                except re.error as e:
+                    log.error("Failed to compile regex /%s/: %s", remain, e)
+                    self.errors[lineno] = str(e)
 
     def skip(self, tag):
         if not self.match_regexes:
@@ -127,8 +112,10 @@ class TagGenreFilter:
                 return True
         return False
 
-    def filter(self, list_of_tags):
-        return list(filterfalse(self.skip, list_of_tags))
+    def filter(self, counter):
+        for name, count in counter:
+            if not self.skip(name):
+                yield (name, count)
 
 
 class TrackArtist(DataObject):
@@ -136,100 +123,136 @@ class TrackArtist(DataObject):
         super().__init__(ta_id)
 
 
-class Track(DataObject, Item):
+class Track(DataObject, FileListItem):
 
     metadata_images_changed = QtCore.pyqtSignal()
 
     def __init__(self, track_id, album=None):
         DataObject.__init__(self, track_id)
-        self.album = album
-        self.linked_files = []
-        self.num_linked_files = 0
+        FileListItem.__init__(self)
         self.metadata = Metadata()
         self.orig_metadata = Metadata()
-        self.error = None
+        self.album = album
+        self.scripted_metadata = Metadata()
         self._track_artists = []
+        self._orig_images = None
+
+    @property
+    def num_linked_files(self):
+        return len(self.files)
 
     def __repr__(self):
         return '<Track %s %r>' % (self.id, self.metadata["title"])
 
-    def add_file(self, file):
-        if file not in self.linked_files:
+    @property
+    def linked_files(self):
+        """For backward compatibility with old code in plugins"""
+        return self.files
+
+    def add_file(self, file, new_album=True):
+        if file not in self.files:
             track_will_expand = self.num_linked_files == 1
-            self.linked_files.append(file)
-            self.num_linked_files += 1
+            if not self.files:  # The track uses original metadata from the file only
+                self._orig_images = self.orig_metadata.images
+                self.orig_metadata.images = ImageList()
+            self.files.append(file)
         self.update_file_metadata(file)
         add_metadata_images(self, [file])
-        self.album._add_file(self, file)
-        file.metadata_images_changed.connect(self.update_orig_metadata_images)
+        self.album.add_file(self, file, new_album=new_album)
+        file.metadata_images_changed.connect(self.update_metadata_images)
         run_file_post_addition_to_track_processors(self, file)
         if track_will_expand:
             # Files get expanded, ensure the existing item renders correctly
-            self.linked_files[0].update_item()
+            self.files[0].update_item()
 
     def update_file_metadata(self, file):
-        if file not in self.linked_files:
+        if file not in self.files:
             return
-        file.copy_metadata(self.metadata)
-        file.metadata['~extension'] = file.orig_metadata['~extension']
+        # Run the scripts for the file to allow usage of
+        # file specific metadata and variables
+        config = get_config()
+        if config.setting["clear_existing_tags"]:
+            metadata = Metadata(self.orig_metadata)
+            metadata_proxy = MultiMetadataProxy(metadata, file.metadata)
+            self.run_scripts(metadata_proxy)
+        else:
+            metadata = Metadata(file.metadata)
+            metadata.update(self.orig_metadata)
+            self.run_scripts(metadata)
+        # Apply changes to the track's metadata done manually after the scripts ran
+        meta_diff = self.metadata.diff(self.scripted_metadata)
+        metadata.update(meta_diff)
+        # Images are not affected by scripting, always use the tracks current images
+        metadata.images = self.metadata.images
+        file.copy_metadata(metadata)
         file.update(signal=False)
         self.update()
 
-    def remove_file(self, file):
-        if file not in self.linked_files:
+    def remove_file(self, file, new_album=True):
+        if file not in self.files:
             return
-        self.linked_files.remove(file)
-        self.num_linked_files -= 1
+        self.files.remove(file)
+        file.metadata_images_changed.disconnect(self.update_metadata_images)
         file.copy_metadata(file.orig_metadata, preserve_deleted=False)
-        file.metadata_images_changed.disconnect(self.update_orig_metadata_images)
-        self.album._remove_file(self, file)
+        self.album.remove_file(self, file, new_album=new_album)
         remove_metadata_images(self, [file])
+        if not self.files and self._orig_images:
+            self.orig_metadata.images = self._orig_images
+            self.metadata.images = self._orig_images.copy()
         run_file_post_removal_from_track_processors(self, file)
         self.update()
         if self.item.isSelected():
             self.tagger.window.refresh_metadatabox()
 
+    @staticmethod
+    def run_scripts(metadata, strip_whitespace=False):
+        for s_name, s_text in enabled_tagger_scripts_texts():
+            parser = ScriptParser()
+            try:
+                parser.eval(s_text, metadata)
+            except ScriptError:
+                log.exception("Failed to run tagger script %s on track", s_name)
+            if strip_whitespace:
+                metadata.strip_whitespace()
+
     def update(self):
         if self.item:
             self.item.update()
 
-    def iterfiles(self, save=False):
-        for file in self.linked_files:
-            yield file
-
     def is_linked(self):
         return self.num_linked_files > 0
 
     def can_save(self):
         """Return if this object can be saved."""
-        for file in self.linked_files:
-            if file.can_save():
-                return True
-        return False
+        return any(file.can_save() for file in self.files)
 
     def can_remove(self):
         """Return if this object can be removed."""
-        for file in self.linked_files:
-            if file.can_remove():
-                return True
-        return False
+        return any(file.can_remove() for file in self.files)
 
     def can_edit_tags(self):
         """Return if this object supports tag editing."""
         return True
 
     def can_view_info(self):
-        return self.num_linked_files == 1 or self.metadata.images
+        return self.num_linked_files == 1 or bool(self.metadata.images)
+
+    @property
+    def can_link_fingerprint(self):
+        """Return True if this item can provide a recording ID for linking to AcoustID."""
+        return True
 
     def column(self, column):
         m = self.metadata
         if column == 'title':
             prefix = "%s-" % m['discnumber'] if m['discnumber'] and m['totaldiscs'] != "1" else ""
             return "%s%s  %s" % (prefix, m['tracknumber'].zfill(2), m['title'])
+        elif column == 'covercount':
+            return self.cover_art_description()
         elif column in m:
             return m[column]
         elif self.num_linked_files == 1:
-            return self.linked_files[0].column(column)
+            return self.files[0].column(column)
 
     def is_video(self):
         return self.metadata['~video'] == '1'
@@ -247,6 +270,7 @@ class Track(DataObject, Item):
         return self.ignored_for_completeness() or self.num_linked_files == 1
 
     def ignored_for_completeness(self):
+        config = get_config()
         if (config.setting['completeness_ignore_videos'] and self.is_video()) \
                 or (config.setting['completeness_ignore_pregap'] and self.is_pregap()) \
                 or (config.setting['completeness_ignore_data'] and self.is_data()) \
@@ -262,6 +286,7 @@ class Track(DataObject, Item):
         return track_artist
 
     def _customize_metadata(self):
+        config = get_config()
         tm = self.metadata
 
         # Custom VA name
@@ -281,60 +306,59 @@ class Track(DataObject, Item):
         if config.setting['convert_punctuation']:
             tm.apply_func(asciipunct)
 
+    @staticmethod
+    def _genres_to_metadata(genres, limit=None, minusage=0, filters='', join_with=None):
+        if limit is not None and limit < 1:
+            return []
+
+        # Ignore tags with zero or lower score
+        genres = +genres
+        if not genres:
+            return []
+
+        # Find most common genres
+        most_common_genres = genres.most_common(limit)
+        topcount = most_common_genres[0][1]
+
+        # Filter by name and usage
+        genres_filter = TagGenreFilter(filters)
+        genres_list = []
+        for name, count in genres_filter.filter(most_common_genres):
+            percent = 100 * count // topcount
+            if percent < minusage:
+                break
+            genres_list.append(name.title())
+        genres_list.sort()
+
+        # And generate the genre metadata tag
+        if join_with:
+            return [join_with.join(genres_list)]
+        else:
+            return genres_list
+
     def _convert_folksonomy_tags_to_genre(self):
-        # Combine release and track tags
-        tags = dict(self.genres)
-        self.merge_genres(tags, self.album.genres)
+        config = get_config()
+        # Combine release and track genres
+        genres = Counter(self.genres)
+        genres += self.album.genres
         if self.album.release_group:
-            self.merge_genres(tags, self.album.release_group.genres)
-        if not tags and config.setting['artists_genres']:
-            # For compilations use each track's artists to look up tags
+            genres += self.album.release_group.genres
+        if not genres and config.setting['artists_genres']:
+            # For compilations use each track's artists to look up genres
             if self.metadata['musicbrainz_albumartistid'] == VARIOUS_ARTISTS_ID:
                 for artist in self._track_artists:
-                    self.merge_genres(tags, artist.genres)
+                    genres += artist.genres
             else:
                 for artist in self.album.get_album_artists():
-                    self.merge_genres(tags, artist.genres)
-        # Ignore tags with zero or lower score
-        tags = dict((name, count) for name, count in tags.items() if count > 0)
-        if not tags:
-            return
-        # Convert counts to values from 0 to 100
-        maxcount = max(tags.values())
-        taglist = []
-        for name, count in tags.items():
-            taglist.append((100 * count // maxcount, name))
-        taglist.sort(reverse=True)
-        # And generate the genre metadata tag
-        maxtags = config.setting['max_genres']
-        minusage = config.setting['min_genre_usage']
-        tag_filter = TagGenreFilter(config.setting['genres_filter'])
-        genre = []
-        for usage, name in taglist[:maxtags]:
-            if tag_filter.skip(name):
-                continue
-            if usage < minusage:
-                break
-            name = _TRANSLATE_TAGS.get(name, name.title())
-            genre.append(name)
-        genre.sort()
-        join_genres = config.setting['join_genres']
-        if join_genres:
-            genre = [join_genres.join(genre)]
-        self.metadata['genre'] = genre
+                    genres += artist.genres
 
-    def update_orig_metadata_images(self):
-        update_metadata_images(self)
-
-    def keep_original_images(self):
-        for file in self.linked_files:
-            file.keep_original_images()
-        self.update_orig_metadata_images()
-        if self.linked_files:
-            self.metadata.images = self.orig_metadata.images.copy()
-        else:
-            self.metadata.images = ImageList()
-        self.update()
+        self.metadata['genre'] = self._genres_to_metadata(
+            genres,
+            limit=config.setting['max_genres'],
+            minusage=config.setting['min_genre_usage'],
+            filters=config.setting['genres_filter'],
+            join_with=config.setting['join_genres']
+        )
 
 
 class NonAlbumTrack(Track):
@@ -359,23 +383,36 @@ class NonAlbumTrack(Track):
     def load(self, priority=False, refresh=False):
         self.metadata.copy(self.album.metadata, copy_images=False)
         self.status = _("[loading recording information]")
-        self.error = None
+        self.clear_errors()
         self.loaded = False
         self.album.update(True)
-        mblogin = False
-        inc = ["artist-credits", "artists", "aliases"]
+        config = get_config()
+        require_authentication = False
+        inc = {
+            "aliases",
+            "artist-credits",
+            "artists",
+        }
         if config.setting["track_ars"]:
-            inc += ["artist-rels", "url-rels", "recording-rels",
-                    "work-rels", "work-level-rels"]
-        mblogin = self.set_genre_inc_params(inc) or mblogin
+            inc |= {
+                "artist-rels",
+                "recording-rels",
+                "url-rels",
+                "work-level-rels",
+                "work-rels",
+            }
+        require_authentication = self.set_genre_inc_params(inc, config) or require_authentication
         if config.setting["enable_ratings"]:
-            mblogin = True
-            inc += ["user-ratings"]
-        self.tagger.mb_api.get_track_by_id(self.id,
-                                           partial(self._recording_request_finished),
-                                           inc, mblogin=mblogin,
-                                           priority=priority,
-                                           refresh=refresh)
+            require_authentication = True
+            inc |= {"user-ratings"}
+        self.tagger.mb_api.get_track_by_id(
+            self.id,
+            self._recording_request_finished,
+            inc=inc,
+            mblogin=require_authentication,
+            priority=priority,
+            refresh=refresh
+        )
 
     def can_remove(self):
         return True
@@ -386,31 +423,23 @@ class NonAlbumTrack(Track):
             return
         try:
             self._parse_recording(recording)
-            for file in self.linked_files:
+            for file in self.files:
                 self.update_file_metadata(file)
         except Exception:
             self._set_error(traceback.format_exc())
 
     def _set_error(self, error):
-        log.error("%r", error)
+        self.error_append(error)
         self.status = _("[could not load recording %s]") % self.id
-        self.error = error
         self.album.update(True)
 
     def _parse_recording(self, recording):
         m = self.metadata
         recording_to_metadata(recording, m, self)
-        self.orig_metadata.copy(m)
         self._customize_metadata()
         run_track_metadata_processors(self.album, m, recording)
-        for s_name, s_text in enabled_tagger_scripts_texts():
-            parser = ScriptParser()
-            try:
-                parser.eval(s_text, m)
-            except ScriptError:
-                log.exception("Failed to run tagger script %s on track", s_name)
-            m.strip_whitespace()
-
+        self.orig_metadata.copy(m)
+        self.run_scripts(m, strip_whitespace=True)
         self.loaded = True
         self.status = None
         if self.callback:
@@ -421,6 +450,9 @@ class NonAlbumTrack(Track):
     def _customize_metadata(self):
         super()._customize_metadata()
         self.metadata['album'] = self.album.metadata['album']
+        if self.metadata['~recording_firstreleasedate']:
+            self.metadata['originaldate'] = self.metadata['~recording_firstreleasedate']
+            self.metadata['originalyear'] = self.metadata['originaldate'][:4]
 
     def run_when_loaded(self, func):
         if self.loaded:
