@@ -4,10 +4,10 @@
 #
 # Copyright (C) 2006-2008 Lukáš Lalinský
 # Copyright (C) 2014 Sophist-UK
-# Copyright (C) 2014, 2018, 2020-2021 Laurent Monin
+# Copyright (C) 2014, 2018, 2020-2022 Laurent Monin
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2019-2022 Philipp Wolfer
+# Copyright (C) 2019-2023 Philipp Wolfer
 # Copyright (C) 2021 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
@@ -40,6 +40,7 @@ from picard.config import (
 )
 from picard.const import DOCS_BASE_URL
 from picard.const.sys import (
+    IS_HAIKU,
     IS_MACOS,
     IS_WIN,
 )
@@ -53,6 +54,8 @@ if IS_MACOS:
     FONT_FAMILY_MONOSPACE = 'Menlo'
 elif IS_WIN:
     FONT_FAMILY_MONOSPACE = 'Consolas'
+elif IS_HAIKU:
+    FONT_FAMILY_MONOSPACE = 'Noto Sans Mono'
 else:
     FONT_FAMILY_MONOSPACE = 'Monospace'
 
@@ -98,7 +101,7 @@ class PreserveGeometry:
         name = widget.objectName()
         if not name:
             name = '.'.join(self._get_lineage(widget))
-            log.debug("Splitter does not have objectName(): %s" % name)
+            log.debug("Splitter does not have objectName(): %s", name)
         return name
 
     @property
@@ -145,7 +148,7 @@ class SingletonDialog:
     def get_instance(cls, *args, **kwargs):
         if not cls._instance:
             cls._instance = cls(*args, **kwargs)
-            cls._instance.finished.connect(cls._on_dialog_finished)
+            cls._instance.destroyed.connect(cls._on_dialog_destroyed)
         return cls._instance
 
     @classmethod
@@ -168,7 +171,7 @@ class SingletonDialog:
         return instance
 
     @classmethod
-    def _on_dialog_finished(cls):
+    def _on_dialog_destroyed(cls):
         cls._instance = None
 
 
