@@ -5,7 +5,7 @@
 # Copyright (C) 2006 Lukáš Lalinský
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2012-2014, 2017, 2019 Wieland Hoffmann
-# Copyright (C) 2013-2014, 2018-2021 Laurent Monin
+# Copyright (C) 2013-2014, 2018-2022 Laurent Monin
 # Copyright (C) 2014, 2017, 2020 Sophist-UK
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016-2017 Sambhav Kothari
@@ -52,6 +52,7 @@ from picard.util import (
     bytes2human,
     encode_filename,
     format_time,
+    open_local_path,
     union_sorted_lists,
 )
 
@@ -271,8 +272,7 @@ class InfoDialog(PicardDialog):
             return
         filename = data.tempfile_filename
         if filename:
-            url = QtCore.QUrl.fromLocalFile(filename)
-            QtGui.QDesktopServices.openUrl(url)
+            open_local_path(filename)
 
 
 def format_file_info(file_):
