@@ -7,14 +7,14 @@
 # Copyright (C) 2008 Gary van der Merwe
 # Copyright (C) 2008 Robert Kaye
 # Copyright (C) 2008 Will
-# Copyright (C) 2008-2010, 2015, 2018-2022 Philipp Wolfer
+# Copyright (C) 2008-2010, 2015, 2018-2023 Philipp Wolfer
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2009 David Hilton
 # Copyright (C) 2011-2012 Chad Wilson
 # Copyright (C) 2011-2013, 2015-2017 Wieland Hoffmann
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2013-2014, 2017 Sophist-UK
-# Copyright (C) 2013-2022 Laurent Monin
+# Copyright (C) 2013-2023 Laurent Monin
 # Copyright (C) 2015 Ohm Patel
 # Copyright (C) 2015 samithaj
 # Copyright (C) 2016 Rahul Raturi
@@ -25,7 +25,7 @@
 # Copyright (C) 2018 Kartik Ohri
 # Copyright (C) 2018 Vishal Choudhary
 # Copyright (C) 2018 virusMac
-# Copyright (C) 2018, 2021 Bob Swift
+# Copyright (C) 2018, 2021-2023 Bob Swift
 # Copyright (C) 2019 Timur Enikeev
 # Copyright (C) 2020-2021 Gabriel Ferreira
 # Copyright (C) 2021 Petit Minion
@@ -49,6 +49,7 @@ from collections import OrderedDict
 from copy import deepcopy
 import datetime
 from functools import partial
+import itertools
 import os.path
 
 from PyQt5 import (
@@ -91,6 +92,7 @@ from picard.util import (
     icontheme,
     iter_files_from_objects,
     iter_unique,
+    open_local_path,
     reconnect,
     restore_method,
     thread,
@@ -98,6 +100,7 @@ from picard.util import (
     webbrowser2,
 )
 from picard.util.cdrom import (
+    DISCID_NOT_LOADED_MESSAGE,
     discid,
     get_cdrom_drives,
 )
@@ -122,15 +125,16 @@ from picard.ui.logview import (
     LogView,
 )
 from picard.ui.metadatabox import MetadataBox
+from picard.ui.newuserdialog import NewUserDialog
 from picard.ui.options.dialog import OptionsDialog
 from picard.ui.passworddialog import (
     PasswordDialog,
     ProxyDialog,
 )
+from picard.ui.savewarningdialog import SaveWarningDialog
 from picard.ui.scripteditor import (
     ScriptEditorDialog,
     ScriptEditorExamples,
-    user_script_title,
 )
 from picard.ui.searchdialog.album import AlbumSearchDialog
 from picard.ui.searchdialog.track import TrackSearchDialog
@@ -187,6 +191,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         super().__init__(parent)
         self.setAttribute(QtCore.Qt.WidgetAttribute.WA_NativeWindow)
         self.__shown = False
+        app = QtCore.QCoreApplication.instance()
+        self._is_wayland = app.is_wayland
         self.selected_objects = []
         self.ignore_selection_changes = IgnoreUpdatesContext(self.update_selection)
         self.toolbar = None
@@ -204,6 +210,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.script_editor_dialog = None
         self.examples = None
 
+        self.check_and_repair_naming_scripts()
         self.check_and_repair_profiles()
 
         self.setupUi()
@@ -292,8 +299,10 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def show(self):
         self.restoreWindowState()
         super().show()
+        self.show_new_user_dialog()
         if self.tagger.autoupdate_enabled:
             self.auto_update_check()
+        self.check_for_plugin_update()
         self.metadata_box.restore_state()
 
     def showEvent(self, event):
@@ -652,10 +661,22 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             action = None
         self.submit_file_as_release_action = action
 
+    @MainWindowActions.add()
+    def _create_similar_items_search_action(self):
+        action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar items..."), self)
+        action.setIconText(_("Similar items"))
+        action.setStatusTip(_("View similar releases or recordings and optionally choose a different one"))
+        action.setEnabled(False)
+        action.setShortcut(QtGui.QKeySequence(_("Ctrl+T")))
+        action.triggered.connect(self.show_similar_items_search)
+        self.similar_items_search_action = action
+
     @MainWindowActions.add()
     def _create_album_search_action(self):
         action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar albums..."), self)
         action.setStatusTip(_("View similar releases and optionally choose a different release"))
+        action.setEnabled(False)
+        action.setShortcut(QtGui.QKeySequence(_("Ctrl+T")))
         action.triggered.connect(self.show_more_albums)
         self.album_search_action = action
 
@@ -734,11 +755,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         action.triggered.connect(self.tagger.lookup_cd)
         self.cd_lookup_action = action
 
-        self.cd_lookup_menu = QtWidgets.QMenu(_("Lookup &CD..."))
-        self.cd_lookup_menu.triggered.connect(self.tagger.lookup_cd)
+        menu = QtWidgets.QMenu(_("Lookup &CD..."))
+        menu.setIcon(icontheme.lookup('media-optical'))
+        menu.triggered.connect(self.tagger.lookup_cd)
+        self.cd_lookup_menu = menu
         if discid is None:
             log.warning("CDROM: discid library not found - Lookup CD functionality disabled")
             self.cd_lookup_action.setEnabled(False)
+            self.cd_lookup_menu.setEnabled(False)
         else:
             thread.run_task(get_cdrom_drives, self._update_cd_lookup_actions)
 
@@ -907,7 +931,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.cd_lookup_menu.clear()
         self.cd_lookup_action.setEnabled(discid is not None)
         if not drives:
-            log.warning("CDROM: No CD-ROM drives found - Lookup CD functionality disabled")
+            log.warning(DISCID_NOT_LOADED_MESSAGE)
         else:
             config = get_config()
             shortcut_drive = config.setting["cd_lookup_device"].split(",")[0] if len(drives) > 1 else ""
@@ -922,7 +946,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _set_cd_lookup_from_file_actions(self, drives):
         if self.cd_lookup_menu.actions():
             self.cd_lookup_menu.addSeparator()
-        action = self.cd_lookup_menu.addAction(_('From EAC / XLD / Whipper &log file...'))
+        action = self.cd_lookup_menu.addAction(_('From CD ripper &log file...'))
         if not drives:
             self._update_cd_lookup_default_action(action)
         action.setData('logfile:eac')
@@ -934,13 +958,15 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             reconnect(self.cd_lookup_action.triggered, self.tagger.lookup_cd)
 
     def _update_cd_lookup_button(self):
-        if self.cd_lookup_menu.actions():
-            button = self.toolbar.widgetForAction(self.cd_lookup_action)
-            if button:
+        button = self.toolbar.widgetForAction(self.cd_lookup_action)
+        enabled = bool(self.cd_lookup_menu.actions() and discid)
+        self.cd_lookup_menu.setEnabled(enabled)
+        if button:
+            if enabled:
                 button.setPopupMode(QtWidgets.QToolButton.ToolButtonPopupMode.MenuButtonPopup)
-            self.cd_lookup_action.setMenu(self.cd_lookup_menu)
-        else:
-            self.cd_lookup_action.setMenu(None)
+                button.setMenu(self.cd_lookup_menu)
+            else:
+                button.setMenu(None)
 
     def toggle_rename_files(self, checked):
         config = get_config()
@@ -1024,12 +1050,12 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         menu.addAction(self.options_action)
         menu = self.menuBar().addMenu(_("&Tools"))
         menu.addAction(self.refresh_action)
-        menu.addAction(self.cd_lookup_action)
+        menu.addMenu(self.cd_lookup_menu)
         menu.addAction(self.autotag_action)
         menu.addAction(self.analyze_action)
         menu.addAction(self.cluster_action)
         menu.addAction(self.browser_lookup_action)
-        menu.addAction(self.track_search_action)
+        menu.addAction(self.similar_items_search_action)
         menu.addAction(self.album_other_versions_action)
         menu.addSeparator()
         menu.addAction(self.generate_fingerprints_action)
@@ -1065,6 +1091,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if self.player:
             self.create_player_toolbar()
         self.create_action_toolbar()
+        self.update_toolbar_style()
 
     def create_action_toolbar(self):
         if self.toolbar:
@@ -1072,10 +1099,11 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.removeToolBar(self.toolbar)
         self.toolbar = toolbar = QtWidgets.QToolBar(_("Actions"))
         self.insertToolBar(self.search_toolbar, self.toolbar)
-        self.update_toolbar_style()
         toolbar.setObjectName("main_toolbar")
+        if self._is_wayland:
+            toolbar.setFloatable(False)  # https://bugreports.qt.io/browse/QTBUG-92191
         if IS_MACOS:
-            self.toolbar.setMovable(False)
+            toolbar.setMovable(False)
 
         def add_toolbar_action(action):
             toolbar.addAction(action)
@@ -1101,6 +1129,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         """"Create a toolbar with internal player control elements"""
         toolbar = self.player.create_toolbar()
         self.addToolBar(QtCore.Qt.ToolBarArea.BottomToolBarArea, toolbar)
+        if self._is_wayland:
+            toolbar.setFloatable(False)  # https://bugreports.qt.io/browse/QTBUG-92191
         self.player_toolbar_toggle_action = toolbar.toggleViewAction()
         toolbar.hide()  # Hide by default
 
@@ -1109,6 +1139,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.search_toolbar = toolbar = self.addToolBar(_("Search"))
         self.search_toolbar_toggle_action = self.search_toolbar.toggleViewAction()
         toolbar.setObjectName("search_toolbar")
+        if self._is_wayland:
+            toolbar.setFloatable(False)  # https://bugreports.qt.io/browse/QTBUG-92191
         if IS_MACOS:
             self.search_toolbar.setMovable(False)
 
@@ -1320,7 +1352,16 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def save(self):
         """Tell the tagger to save the selected objects."""
-        self.tagger.save(self.selected_objects)
+        config = get_config()
+        if config.setting["file_save_warning"]:
+            count = len(self.tagger.get_files_from_objects(self.selected_objects))
+            msg = SaveWarningDialog(self, count)
+            proceed_with_save, disable_warning = msg.show()
+            config.setting["file_save_warning"] = not disable_warning
+        else:
+            proceed_with_save = True
+        if proceed_with_save:
+            self.tagger.save(self.selected_objects)
 
     def remove(self):
         """Tell the tagger to remove the selected objects."""
@@ -1338,12 +1379,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 self.tagger.generate_fingerprints(self.selected_objects)
         self._ensure_fingerprinting_configured(callback)
 
-    def _openUrl(self, url):
-        return QtCore.QUrl.fromLocalFile(url)
-
     def play_file(self):
         for file in iter_files_from_objects(self.selected_objects):
-            QtGui.QDesktopServices.openUrl(self._openUrl(file.filename))
+            open_local_path(file.filename)
 
     def _on_player_error(self, error, msg):
         self.set_statusbar_message(msg, echo=log.warning, translate=None)
@@ -1353,7 +1391,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             os.path.dirname(f.filename) for f
             in iter_files_from_objects(self.selected_objects))
         for folder in folders:
-            QtGui.QDesktopServices.openUrl(self._openUrl(folder))
+            open_local_path(folder)
 
     def _ensure_fingerprinting_configured(self, callback):
         config = get_config()
@@ -1382,6 +1420,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 return obj
         return None
 
+    def show_similar_items_search(self):
+        obj = self.get_first_obj_with_type(Cluster)
+        if obj:
+            self.show_more_albums()
+        else:
+            self.show_more_tracks()
+
     def show_more_tracks(self):
         if not self.selected_objects:
             return
@@ -1389,7 +1434,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if isinstance(obj, Track) and obj.files:
             obj = obj.files[0]
         if not isinstance(obj, File):
-            log.debug('show_more_tracks expected a File, got %r' % obj)
+            log.debug('show_more_tracks expected a File, got %r', obj)
             return
         dialog = TrackSearchDialog(self, force_advanced_search=True)
         dialog.show_similar_tracks(obj)
@@ -1398,7 +1443,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def show_more_albums(self):
         obj = self.get_first_obj_with_type(Cluster)
         if not obj:
-            log.debug('show_more_albums expected a Cluster, got %r' % obj)
+            log.debug('show_more_albums expected a Cluster, got %r', obj)
             return
         dialog = AlbumSearchDialog(self, force_advanced_search=True)
         dialog.show_similar_albums(obj)
@@ -1431,7 +1476,19 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         dialog.exec_()
 
     def cluster(self):
-        self.tagger.cluster(self.selected_objects, self._cluster_finished)
+        # Cluster all selected unclustered files. If there are no selected
+        # unclustered files cluster all unclustered files.
+        files = (
+            f for f in iter_files_from_objects(self.selected_objects)
+            if f.parent == self.tagger.unclustered_files
+        )
+        try:
+            file = next(files)
+        except StopIteration:
+            files = self.tagger.unclustered_files.files
+        else:
+            files = itertools.chain([file], files)
+        self.tagger.cluster(files, self._cluster_finished)
 
     def _cluster_finished(self):
         self.panel.update_current_view()
@@ -1495,6 +1552,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         can_browser_lookup = bool(single and single.can_browser_lookup())
         is_file = bool(single and isinstance(single, (File, Track)))
         is_album = bool(single and isinstance(single, Album))
+        is_cluster = bool(single and isinstance(single, Cluster) and not single.special)
 
         if not self.selected_objects:
             have_objects = have_files = False
@@ -1545,7 +1603,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.submit_file_as_release_action.setEnabled(have_files)
         files = self.get_selected_or_unmatched_files()
         self.tags_from_filenames_action.setEnabled(bool(files))
+        self.similar_items_search_action.setEnabled(is_file or is_cluster)
         self.track_search_action.setEnabled(is_file)
+        self.album_search_action.setEnabled(is_cluster)
         self.album_other_versions_action.setEnabled(is_album)
 
     def update_selection(self, objects=None, new_selection=True, drop_album_caches=False):
@@ -1719,14 +1779,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         update_level = config.setting['update_level']
         today = datetime.date.today().toordinal()
         do_auto_update_check = check_for_updates and update_check_days > 0 and today >= last_update_check + update_check_days
-        log.debug('{check_status} start-up check for program updates.  Today: {today_date}, Last check: {last_check} (Check interval: {check_interval} days), Update level: {update_level} ({update_level_name})'.format(
-            check_status='Initiating' if do_auto_update_check else 'Skipping',
-            today_date=datetime.date.today(),
-            last_check=str(datetime.date.fromordinal(last_update_check)) if last_update_check > 0 else 'never',
-            check_interval=update_check_days,
-            update_level=update_level,
-            update_level_name=PROGRAM_UPDATE_LEVELS[update_level]['name'] if update_level in PROGRAM_UPDATE_LEVELS else 'unknown',
-        ))
+        log.debug('%(check_status)s startup check for program updates.  Today: %(today_date)s, Last check: %(last_check)s (Check interval: %(check_interval)s days), Update level: %(update_level)s (%(update_level_name)s)', {
+            'check_status': 'Initiating' if do_auto_update_check else 'Skipping',
+            'today_date': datetime.date.today(),
+            'last_check': str(datetime.date.fromordinal(last_update_check)) if last_update_check > 0 else 'never',
+            'check_interval': update_check_days,
+            'update_level': update_level,
+            'update_level_name': PROGRAM_UPDATE_LEVELS[update_level]['name'] if update_level in PROGRAM_UPDATE_LEVELS else 'unknown',
+        })
         if do_auto_update_check:
             self.check_for_update(False)
 
@@ -1738,6 +1798,23 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             callback=update_last_check_date
         )
 
+    def check_and_repair_naming_scripts(self):
+        """Check the 'file_renaming_scripts' config setting to ensure that the list of scripts
+        is not empty.  Check that the 'selected_file_naming_script_id' config setting points to
+        a valid file naming script.
+        """
+        config = get_config()
+        script_key = 'file_renaming_scripts'
+        if not config.setting[script_key]:
+            config.setting[script_key] = {
+                script['id']: script.to_dict()
+                for script in get_file_naming_script_presets()
+            }
+        naming_script_ids = list(config.setting[script_key])
+        script_id_key = 'selected_file_naming_script_id'
+        if config.setting[script_id_key] not in naming_script_ids:
+            config.setting[script_id_key] = naming_script_ids[0]
+
     def check_and_repair_profiles(self):
         """Check the profiles and profile settings and repair the values if required.
         Checks that there is a settings dictionary for each profile, and that no profiles
@@ -1747,7 +1824,6 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config = get_config()
         naming_scripts = config.setting["file_renaming_scripts"]
         naming_script_ids = set(naming_scripts.keys())
-        naming_script_ids |= set(item["id"] for item in get_file_naming_script_presets())
         profile_settings = deepcopy(config.profiles[SettingConfigSection.SETTINGS_KEY])
         for profile in config.profiles[SettingConfigSection.PROFILES_KEY]:
             p_id = profile["id"]
@@ -1796,11 +1872,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             group.addAction(script_action)
 
         for (id, naming_script) in sorted(naming_scripts.items(), key=lambda item: item[1]['title']):
-            _add_menu_item(user_script_title(naming_script['title']), id)
-
-        # Add preset scripts not provided in the user-defined scripts list.
-        for script_item in get_file_naming_script_presets():
-            _add_menu_item(script_item['title'], script_item['id'])
+            _add_menu_item(naming_script['title'], id)
 
     def select_new_naming_script(self, id):
         """Update the currently selected naming script ID in the settings.
@@ -1915,6 +1987,20 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 self.make_script_selector_menu()
                 return
 
+    def show_new_user_dialog(self):
+        config = get_config()
+        if config.setting["show_new_user_dialog"]:
+            msg = NewUserDialog(self)
+            config.setting["show_new_user_dialog"] = msg.show()
+
+    def check_for_plugin_update(self):
+        config = get_config()
+        if config.setting['check_for_plugin_updates']:
+            self.tagger.pluginmanager.check_update(self)
+
+    def show_plugins_options_page(self):
+        self.show_options(page='plugins')
+
 
 def update_last_check_date(is_success):
     if is_success:
