@@ -6,11 +6,11 @@
 # Copyright (C) 2006-2007, 2012 Lukáš Lalinský
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2012 Nikolai Prokoschenko
-# Copyright (C) 2013-2014, 2017-2021 Laurent Monin
+# Copyright (C) 2013-2014, 2017-2022 Laurent Monin
 # Copyright (C) 2013-2014, 2021 Sophist-UK
 # Copyright (C) 2015 Ohm Patel
 # Copyright (C) 2015 Wieland Hoffmann
-# Copyright (C) 2015, 2018-2021 Philipp Wolfer
+# Copyright (C) 2015, 2018-2022 Philipp Wolfer
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
 # Copyright (C) 2020 Felix Schwarz
@@ -55,6 +55,7 @@ from picard.file import File
 from picard.metadata import MULTI_VALUED_JOINER
 from picard.track import Track
 from picard.util import (
+    IgnoreUpdatesContext,
     format_time,
     icontheme,
     restore_method,
@@ -197,7 +198,7 @@ class TableTagEditorDelegate(TagEditorDelegate):
         return QtCore.QSize(size_hint.width(), min(160, size_hint.height()))
 
     def get_tag_name(self, index):
-        return index.data(QtCore.Qt.UserRole)
+        return index.data(QtCore.Qt.ItemDataRole.UserRole)
 
 
 class MetadataBox(QtWidgets.QTableWidget):
@@ -219,15 +220,15 @@ class MetadataBox(QtWidgets.QTableWidget):
         self.setColumnCount(3)
         self.setHorizontalHeaderLabels((_("Tag"), _("Original Value"), _("New Value")))
         self.horizontalHeader().setStretchLastSection(True)
-        self.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.Stretch)
+        self.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeMode.Stretch)
         self.horizontalHeader().setSectionsClickable(False)
         self.verticalHeader().setDefaultSectionSize(21)
         self.verticalHeader().setVisible(False)
-        self.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
-        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
+        self.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollMode.ScrollPerPixel)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
         self.setTabKeyNavigation(False)
         self.setStyleSheet("QTableWidget {border: none;}")
-        self.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 1)
+        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_MacShowFocusRect, 1)
         self.setItemDelegate(TableTagEditorDelegate(self))
         self.setWordWrap(False)
         self.files = set()
@@ -253,6 +254,7 @@ class MetadataBox(QtWidgets.QTableWidget):
         self.preserved_tags = PreservedTags()
         self._single_file_album = False
         self._single_track_album = False
+        self.ignore_updates = IgnoreUpdatesContext(onexit=self.update)
         self.tagger.clipboard().dataChanged.connect(self.update_clipboard)
 
     def get_file_lookup(self):
@@ -287,10 +289,10 @@ class MetadataBox(QtWidgets.QTableWidget):
         if index.column() != self.COLUMN_NEW:
             return False
         item = self.itemFromIndex(index)
-        if item.flags() & QtCore.Qt.ItemIsEditable and \
-           trigger in {QtWidgets.QAbstractItemView.DoubleClicked,
-                       QtWidgets.QAbstractItemView.EditKeyPressed,
-                       QtWidgets.QAbstractItemView.AnyKeyPressed}:
+        if item.flags() & QtCore.Qt.ItemFlag.ItemIsEditable and \
+           trigger in {QtWidgets.QAbstractItemView.EditTrigger.DoubleClicked,
+                       QtWidgets.QAbstractItemView.EditTrigger.EditKeyPressed,
+                       QtWidgets.QAbstractItemView.EditTrigger.AnyKeyPressed}:
             tag = self.tag_diff.tag_names[item.row()]
             values = self.tag_diff.new[tag]
             if len(values) > 1:
@@ -303,9 +305,9 @@ class MetadataBox(QtWidgets.QTableWidget):
         return False
 
     def keyPressEvent(self, event):
-        if event.matches(QtGui.QKeySequence.Copy):
+        if event.matches(QtGui.QKeySequence.StandardKey.Copy):
             self.copy_value()
-        elif event.matches(QtGui.QKeySequence.Paste):
+        elif event.matches(QtGui.QKeySequence.StandardKey.Paste):
             self.paste_value()
         else:
             super().keyPressEvent(event)
@@ -436,11 +438,11 @@ class MetadataBox(QtWidgets.QTableWidget):
                     menu.addSeparator()
                     copy_action = QtWidgets.QAction(icontheme.lookup('edit-copy', icontheme.ICON_SIZE_MENU), _("&Copy"), self)
                     copy_action.triggered.connect(self.copy_value)
-                    copy_action.setShortcut(QtGui.QKeySequence.Copy)
+                    copy_action.setShortcut(QtGui.QKeySequence.StandardKey.Copy)
                     menu.addAction(copy_action)
                     paste_action = QtWidgets.QAction(icontheme.lookup('edit-paste', icontheme.ICON_SIZE_MENU), _("&Paste"), self)
                     paste_action.triggered.connect(self.paste_value)
-                    paste_action.setShortcut(QtGui.QKeySequence.Paste)
+                    paste_action.setShortcut(QtGui.QKeySequence.StandardKey.Paste)
                     paste_action.setEnabled(editable)
                     menu.addAction(paste_action)
             if single_tag or removals or useorigs:
@@ -537,10 +539,10 @@ class MetadataBox(QtWidgets.QTableWidget):
 
     @throttle(100)
     def update(self, drop_album_caches=False):
-        if self.editing:
-            return
         new_selection = self.selection_dirty
-        if self.selection_dirty:
+        if self.editing or (self.ignore_updates and not new_selection):
+            return
+        if new_selection:
             self._update_selection()
         thread.run_task(partial(self._update_tags, new_selection, drop_album_caches), self._update_items,
             thread_pool=self.tagger.priority_thread_pool)
@@ -573,7 +575,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             return self.tag_diff
 
         self.colors = {
-            TagStatus.NOCHANGE: self.palette().color(QtGui.QPalette.Text),
+            TagStatus.NOCHANGE: self.palette().color(QtGui.QPalette.ColorRole.Text),
             TagStatus.REMOVED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_removed')),
             TagStatus.ADDED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_added')),
             TagStatus.CHANGED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_changed'))
@@ -660,8 +662,8 @@ class MetadataBox(QtWidgets.QTableWidget):
 
         self.setRowCount(len(result.tag_names))
 
-        orig_flags = QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
-        new_flags = orig_flags | QtCore.Qt.ItemIsEditable
+        orig_flags = QtCore.Qt.ItemFlag.ItemIsSelectable | QtCore.Qt.ItemFlag.ItemIsEnabled
+        new_flags = orig_flags | QtCore.Qt.ItemFlag.ItemIsEditable
 
         for i, name in enumerate(result.tag_names):
             tag_item = self.item(i, 0)
@@ -702,7 +704,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             orig_item.setForeground(color)
             new_item.setForeground(color)
 
-            alignment = QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop
+            alignment = QtCore.Qt.AlignmentFlag.AlignLeft | QtCore.Qt.AlignmentFlag.AlignTop
             tag_item.setTextAlignment(alignment)
             orig_item.setTextAlignment(alignment)
             new_item.setTextAlignment(alignment)
@@ -712,7 +714,7 @@ class MetadataBox(QtWidgets.QTableWidget):
 
     def set_item_value(self, item, tags, name):
         text, italic = tags.display_value(name)
-        item.setData(QtCore.Qt.UserRole, name)
+        item.setData(QtCore.Qt.ItemDataRole.UserRole, name)
         item.setText(text)
         font = item.font()
         font.setItalic(italic)
@@ -724,7 +726,7 @@ class MetadataBox(QtWidgets.QTableWidget):
         state = config.persist["metadatabox_header_state"]
         header = self.horizontalHeader()
         header.restoreState(state)
-        header.setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
+        header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeMode.Interactive)
 
     def save_state(self):
         config = get_config()
