@@ -8,13 +8,13 @@
 # Copyright (C) 2011 Pavan Chander
 # Copyright (C) 2011-2012, 2019 Wieland Hoffmann
 # Copyright (C) 2011-2013 Michael Wiencek
-# Copyright (C) 2013, 2017-2021 Laurent Monin
+# Copyright (C) 2013, 2017-2022 Laurent Monin
 # Copyright (C) 2014 Sophist-UK
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Suhas
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2021-2022 Bob Swift
 # Copyright (C) 2021 Gabriel Ferreira
+# Copyright (C) 2021-2022 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -65,6 +65,7 @@ from picard.ui.options import (  # noqa: F401 # pylint: disable=unused-import
     genres,
     interface,
     interface_colors,
+    interface_toolbar,
     interface_top_tags,
     maintenance,
     matching,
@@ -75,6 +76,7 @@ from picard.ui.options import (  # noqa: F401 # pylint: disable=unused-import
     ratings,
     releases,
     renaming,
+    renaming_compat,
     scripting,
     tags,
     tags_compatibility_aac,
@@ -94,6 +96,8 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         ListOption("persist", "options_pages_tree_state", []),
     ]
 
+    suspend_signals = False
+
     def add_pages(self, parent, default_page, parent_item):
         pages = [(p.SORT_ORDER, p.NAME, p) for p in self.pages if p.PARENT == parent]
         items = []
@@ -152,9 +156,10 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         for Page in page_classes:
             try:
                 page = Page(self.ui.pages_stack)
+                page.set_dialog(self)
                 self.pages.append(page)
             except Exception:
-                log.exception('Failed initializing options page %r', page)
+                log.exception('Failed initializing options page %r', Page)
         self.item_to_page = {}
         self.page_to_item = {}
         self.default_item = None
@@ -240,7 +245,8 @@ class OptionsDialog(PicardDialog, SingletonDialog):
         return _('Unknown profile')
 
     def update_from_profile_changes(self):
-        self.highlight_enabled_profile_options(load_settings=True)
+        if not self.suspend_signals:
+            self.highlight_enabled_profile_options(load_settings=True)
 
     def get_working_profile_data(self):
         profile_page = self.get_page('profiles')
@@ -408,13 +414,18 @@ class OptionsDialog(PicardDialog, SingletonDialog):
                 item.setExpanded(is_expanded)
 
     def restore_all_defaults(self):
+        self.suspend_signals = True
         for page in self.pages:
-            page.restore_defaults()
-        self.highlight_enabled_profile_options()
+            try:
+                page.restore_defaults()
+            except Exception as e:
+                log.error('Failed restoring all defaults for page %r: %s', page, e)
+        self.highlight_enabled_profile_options(load_settings=False)
+        self.suspend_signals = False
 
     def restore_page_defaults(self):
         self.ui.pages_stack.currentWidget().restore_defaults()
-        self.highlight_enabled_profile_options()
+        self.highlight_enabled_profile_options(load_settings=False)
 
     def confirm_reset(self):
         msg = _("You are about to reset your options for this page.")
