@@ -3,13 +3,13 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2007 Lukáš Lalinský
-# Copyright (C) 2009, 2014, 2019-2020 Philipp Wolfer
+# Copyright (C) 2009, 2014, 2019-2022 Philipp Wolfer
 # Copyright (C) 2012-2013 Michael Wiencek
 # Copyright (C) 2014, 2017 Sophist-UK
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Ville Skyttä
-# Copyright (C) 2018 Laurent Monin
 # Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2018, 2020-2022 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -98,7 +98,7 @@ class TagMatchExpression:
 
 class TagsFromFileNamesDialog(PicardDialog):
 
-    autorestore = False
+    help_url = 'doc_tags_from_filenames'
 
     options = [
         TextOption("persist", "tags_from_filenames_format", ""),
@@ -108,7 +108,6 @@ class TagsFromFileNamesDialog(PicardDialog):
         super().__init__(parent)
         self.ui = Ui_TagsFromFileNamesDialog()
         self.ui.setupUi(self)
-        self.restore_geometry()
         items = [
             "%artist%/%album%/%title%",
             "%artist%/%album%/%tracknumber% %title%",
@@ -128,10 +127,12 @@ class TagsFromFileNamesDialog(PicardDialog):
             selected_index = items.index(tff_format)
         self.ui.format.addItems(items)
         self.ui.format.setCurrentIndex(selected_index)
-        self.ui.buttonbox.addButton(StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.AcceptRole)
-        self.ui.buttonbox.addButton(StandardButton(StandardButton.CANCEL), QtWidgets.QDialogButtonBox.RejectRole)
+        self.ui.buttonbox.addButton(StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.ButtonRole.HelpRole)
+        self.ui.buttonbox.addButton(StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.ButtonRole.AcceptRole)
+        self.ui.buttonbox.addButton(StandardButton(StandardButton.CANCEL), QtWidgets.QDialogButtonBox.ButtonRole.RejectRole)
         self.ui.buttonbox.accepted.connect(self.accept)
         self.ui.buttonbox.rejected.connect(self.reject)
+        self.ui.buttonbox.helpRequested.connect(self.show_help)
         self.ui.preview.clicked.connect(self.preview)
         self.ui.files.setHeaderLabels([_("File Name")])
         self.files = files
@@ -152,15 +153,14 @@ class TagsFromFileNamesDialog(PicardDialog):
             for i, column in enumerate(columns):
                 values = matches.get(column, [])
                 item.setText(i + 1, '; '.join(values))
-        self.ui.files.header().resizeSections(QtWidgets.QHeaderView.ResizeToContents)
+        self.ui.files.header().resizeSections(QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
         self.ui.files.header().setStretchLastSection(True)
 
     def accept(self):
         expression = TagMatchExpression(self.ui.format.currentText(), self.ui.replace_underscores.isChecked())
         for file in self.files:
             metadata = expression.match_file(file.filename)
-            for name, values in metadata.items():
-                file.metadata[name] = values
+            file.metadata.update(metadata)
             file.update()
         config = get_config()
         config.persist["tags_from_filenames_format"] = self.ui.format.currentText()
