@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2007 Lukáš Lalinský
+# Copyright (C) 2013-2015, 2018-2022 Laurent Monin
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2017 Frederik “Freso” S. Olesen
+# Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2019, 2021-2022 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,19 +24,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import sys
 
 from PyQt5 import (
     QtCore,
-    QtGui,
     QtWidgets,
 )
 
-from picard import config
-from picard.util import (
-    find_existing_path,
-    icontheme,
+from picard.config import get_config
+from picard.const.sys import (
+    IS_MACOS,
+    IS_WIN,
 )
+from picard.util import find_existing_path
 
 
 class StandardButton(QtWidgets.QPushButton):
@@ -49,7 +55,7 @@ class StandardButton(QtWidgets.QPushButton):
     def __init__(self, btntype):
         label = _(self.__types[btntype][0])
         args = [label]
-        if sys.platform != 'win32' and sys.platform != 'darwin':
+        if not IS_WIN and not IS_MACOS:
             iconname = self.__types[btntype][1]
             if hasattr(QtWidgets.QStyle, iconname):
                 icon = self.tagger.style().standardIcon(getattr(QtWidgets.QStyle, iconname))
@@ -57,18 +63,8 @@ class StandardButton(QtWidgets.QPushButton):
         super().__init__(*args)
 
 
-# The following code is there to fix
-# https://tickets.metabrainz.org/browse/PICARD-417
-# In some older version of PyQt/sip it's impossible to connect a signal
-# emitting an `int` to a slot expecting a `bool`.
-# By using `enabledSlot` instead we can force python to do the
-# conversion from int (`state`) to bool.
-def enabledSlot(func, state):
-    """Calls `func` with `state`."""
-    func(state)
-
-
 def find_starting_directory():
+    config = get_config()
     if config.setting["starting_directory"]:
         path = config.setting["starting_directory_path"]
     else:
@@ -87,9 +83,8 @@ class MultiDirsSelectDialog(QtWidgets.QFileDialog):
         super().__init__(*args)
         self.setFileMode(self.Directory)
         self.setOption(self.ShowDirsOnly)
-        if sys.platform in ["darwin", "win32"]:
-            # The native dialog doesn't allow selecting >1 directory
-            self.setOption(self.DontUseNativeDialog)
+        # The native dialog doesn't allow selecting >1 directory
+        self.setOption(self.DontUseNativeDialog)
         for view in self.findChildren((QtWidgets.QListView, QtWidgets.QTreeView)):
             if isinstance(view.model(), QtWidgets.QFileSystemModel):
-                view.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
+                view.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
