@@ -5,14 +5,15 @@
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2008 Will
-# Copyright (C) 2008, 2018-2020 Philipp Wolfer
+# Copyright (C) 2008, 2018-2021 Philipp Wolfer
 # Copyright (C) 2009 david
 # Copyright (C) 2013 Johannes Dewender
 # Copyright (C) 2013 Sebastian Ramacher
-# Copyright (C) 2013, 2018-2019 Laurent Monin
+# Copyright (C) 2013, 2018-2021 Laurent Monin
 # Copyright (C) 2013-2014 Michael Wiencek
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2022 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,21 +30,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from PyQt5.QtCore import (
-    QFile,
-    QIODevice,
-)
+import os.path
 
+from picard import log
 from picard.config import get_config
 from picard.const.sys import (
     IS_LINUX,
     IS_WIN,
 )
-from picard.util import uniqify
-
-
-if IS_WIN:
-    from ctypes import windll
 
 
 try:
@@ -54,6 +48,7 @@ except ImportError:
     except (ImportError, OSError):
         discid = None
 
+DISCID_NOT_LOADED_MESSAGE = "CDROM: discid library not found - Lookup CD functionality disabled"
 
 DEFAULT_DRIVES = []
 if discid is not None:
@@ -63,67 +58,76 @@ if discid is not None:
 
 LINUX_CDROM_INFO = '/proc/sys/dev/cdrom/info'
 
-# if get_cdrom_drives() lists ALL drives available on the machine
+
+def _generic_iter_drives():
+    config = get_config()
+    yield from (
+        device.strip() for device
+        in config.setting["cd_lookup_device"].split(",")
+        if device and not device.isspace()
+    )
+
+
+def _parse_linux_cdrom_info(f):
+    drive_names = []
+    drive_audio_caps = []
+    DRIVE_NAME = 'drive name:'
+    CAN_PLAY_AUDIO = 'Can play audio:'
+    for line in f:
+        if line.startswith(DRIVE_NAME):
+            drive_names = line[len(DRIVE_NAME):].split()
+            break
+    if drive_names:
+        for line in f:
+            if line.startswith(CAN_PLAY_AUDIO):
+                drive_audio_caps = [v == '1' for v in line[len(CAN_PLAY_AUDIO):].split()]
+                break
+    yield from zip(drive_names, drive_audio_caps)
+
+
 if IS_WIN:
+    from ctypes import windll
+
     AUTO_DETECT_DRIVES = True
-elif IS_LINUX and QFile.exists(LINUX_CDROM_INFO):
+    DRIVE_TYPE_CDROM = 5
+
+    def _iter_drives():
+        GetLogicalDrives = windll.kernel32.GetLogicalDrives
+        GetDriveType = windll.kernel32.GetDriveTypeW
+        mask = GetLogicalDrives()
+        for i in range(26):
+            if mask >> i & 1:
+                drive = chr(i + ord("A")) + ":"
+                if GetDriveType(drive) == DRIVE_TYPE_CDROM:
+                    yield drive
+
+elif IS_LINUX and os.path.isfile(LINUX_CDROM_INFO):
     AUTO_DETECT_DRIVES = True
+
+    def _iter_drives():
+        # Read info from /proc/sys/dev/cdrom/info
+        with open(LINUX_CDROM_INFO, 'r') as f:
+            # Show only drives that are capable of playing audio
+            yield from (
+                os.path.realpath('/dev/%s' % drive)
+                for drive, can_play_audio in _parse_linux_cdrom_info(f)
+                if can_play_audio
+            )
+
 else:
     # There might be more drives we couldn't detect
     # setting uses a text field instead of a drop-down
     AUTO_DETECT_DRIVES = False
+    _iter_drives = _generic_iter_drives
 
 
 def get_cdrom_drives():
     """List available disc drives on the machine
     """
     # add default drive from libdiscid to the list
-    drives = list(DEFAULT_DRIVES)
-
-    if IS_WIN:
-        GetLogicalDrives = windll.kernel32.GetLogicalDrives
-        GetDriveType = windll.kernel32.GetDriveTypeW
-        DRIVE_CDROM = 5
-        mask = GetLogicalDrives()
-        for i in range(26):
-            if mask >> i & 1:
-                drive = chr(i + ord("A")) + ":"
-                if GetDriveType(drive) == DRIVE_CDROM:
-                    drives.append(drive)
-
-    elif IS_LINUX and AUTO_DETECT_DRIVES:
-        # Read info from /proc/sys/dev/cdrom/info
-        cdinfo = QFile(LINUX_CDROM_INFO)
-        if cdinfo.open(QIODevice.ReadOnly | QIODevice.Text):
-            drive_names = []
-            drive_audio_caps = []
-            while True:
-                line = bytes(cdinfo.readLine()).decode()
-                if not line:
-                    break
-                if ":" in line:
-                    key, values = line.split(':')
-                    if key == 'drive name':
-                        drive_names = values.split()
-                    elif key == 'Can play audio':
-                        drive_audio_caps = [v == '1' for v in values.split()]
-                        break  # no need to continue past this line
-            # Show only drives that are capable of playing audio
-            for index, drive in enumerate(drive_names):
-                if drive_audio_caps[index]:
-                    device = '/dev/%s' % drive
-                    symlink_target = QFile.symLinkTarget(device)
-                    if symlink_target != '':
-                        device = symlink_target
-                    drives.append(device)
-
-    else:
-        config = get_config()
-        for device in config.setting["cd_lookup_device"].split(","):
-            # Need to filter out empty strings,
-            # particularly if the device list is empty
-            if device.strip() != '':
-                drives.append(device.strip())
-
-    # make sure no drive is listed twice (given by multiple sources)
-    return sorted(uniqify(drives))
+    drives = set(DEFAULT_DRIVES)
+    try:
+        drives |= set(_iter_drives())
+    except OSError as error:
+        log.error(error)
+    return sorted(drives)
